from pylel.tools import get_real_dir_name
from .evaluate_expr import evaluate_expr
from .scope import Scope

def evaluate(ast, base_path=None):
	if not base_path:
		base_path = get_real_dir_name(__file__)
	root_scope = Scope(None, base_path)
	return [evaluate_expr(root_scope, expr) for expr in ast]



from .lel_boolean import LEL_BOOLEAN
from .lel_general import LEL_GENERAL
from .lel_list import LEL_LIST
from .lel_math import LEL_MATH
from .lel_string import LEL_STRING
from .lel_bit import LEL_BIT

STANDARD = LEL_GENERAL.copy()
STANDARD.update(LEL_BOOLEAN)
STANDARD.update(LEL_MATH)
STANDARD.update(LEL_LIST)
STANDARD.update(LEL_STRING)
STANDARD.update(LEL_STRING)
STANDARD.update(LEL_BIT)



from __future__ import absolute_import
import math as pymath
from pylel.token import Symbols, Token

def _sin(x_degree):
	if x_degree.type != Symbols.NUMBER:
		raise Exception("sin only operates on NUMBER type")
	return Token(Symbols.NUMBER, pymath.sin(x_degree.value))

def _cos(x_degree):
	if x_degree.type != Symbols.NUMBER:
		raise Exception("sin only operates on NUMBER type")
	return Token(Symbols.NUMBER, pymath.cos(x_degree.value))

METHODS = {
	"sin": _sin,
	"cos": _cos,
}



import os
from require import require
from pylel.tools import get_real_dir_name
from pylel.libraries import LIBRARIES

LIBRARY_LOCATION = "../../../libraries/"
LIBRARY_EXTENSION = ".py"
LIBRARY_SEPARATOR = "-"

def lel_use(evaluate_expr, scope, expr):
	if len(expr) != 2:
		raise Exception("Use requires only 1 argument. Got {} at expression {}"\
			.format(len(expr), expr))
	library = os.path.join(get_real_dir_name(__file__),\
	LIBRARY_LOCATION, expr[1].value + LIBRARY_EXTENSION)
	library_methods = require(str(library)).METHODS
	LIBRARIES.update({
		expr[1].value + LIBRARY_SEPARATOR + k: library_methods[k] \
		for k in library_methods.keys()
	})
